#!/bin/sh
#@***h* GPCP_Daily_Analysis/GPCPcdr_ECPd_Process1month.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   GPCPcdr_ECPd_Process1month.sh
#
# PURPOSE
#   This routine produces the daily GPCP NetCDF file.
#
# DESCRIPTION
# Executive Control Package - Daily Main driver code to process a single month
# This code runs all packages for a given year/month. It checks that all data is 
# present and then calls each of the constituent parts of the processing code.
#
# In its simplest form, this code can be used to process
# all data for a single month by running 
#   GPCPcdr_ECPd_Process1month.sh YYYY MM
# 
# The function will also accept input flags that control processing.  
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 1/2017 Initial Version prepared for NCEI
#
#@*****

# check number if inputs
if [ $# -lt 2 ]; then
    echo "Incorrect number of inputs; need: yyyy mm"
    exit 0
elif [ $# -gt 9 ]; then
    echo "Max number of inputs (9) exceeded"
    exit 0
fi


#############################################
# set inputs parameters and processing flags

# Default flags for processing
# these are overwritten if provided as input arguments
flag_GPCPcdr_d1_GetInputFiles=1
flag_GPCPcdr_d2_GPROFv2004=1
flag_GPCPcdr_d4_CORR_G=1
flag_GPCPcdr_d4_ADJIR=1
flag_GPCPcdr_d4_CORR_T=1
flag_GPCPcdr_d4_FUSE2=1
flag_GPCPcdr_d4_WriteNC=1

# set year and month
yyyy=$1
mm=$2

# get two digit year
yy=`echo $yyyy | awk '{print substr($0, 3, 2)}'`


# adjust flags based on inputs
if [ $# -ge "3" ] ; then
    flag_GPCPcdr_d1_GetInputFiles=$3
fi
if [ $# -ge "4" ] ; then
    flag_GPCPcdr_d2_GPROFv2004=$4
fi
if [ $# -ge "5" ] ; then
    flag_GPCPcdr_d4_CORR_G=$5
fi
if [ $# -ge "6" ] ; then
    flag_GPCPcdr_d4_ADJIR=$6
fi
if [ $# -ge "7" ] ; then
    flag_GPCPcdr_d4_CORR_T=$7
fi
if [ $# -ge "8" ] ; then
    flag_GPCPcdr_d4_FUSE2=$8
fi
if [ $# -ge "9" ] ; then
    flag_GPCPcdr_d4_WriteNC=$9
fi




#############################################
# Some settings 

# read data directories from OptionsM.h
InputPATH=$(echo `grep "InputPATH" OptionsD.h` | awk -F\"  '{print $2}')
InterimPATH=$(
echo `grep "InterimPATH" OptionsD.h` | awk -F\"  '{print $2}')
OutputPATH=$(echo `grep "OutputPATH" OptionsD.h` | awk -F\"  '{print $2}')
GPROFPATH=$(echo `grep "GPROFPATH" OptionsD.h` | awk -F\"  '{print $2}')


# set the satelite for SSMI/SSMIS
# From GPCP tech doc (1 August 2012, p26)
# The SSMI estimates are based on data from the F08 
# instrument from mid-1987 through 1991, with the F11 
# being used for 1992 through April 1995, and the F13 
# for May 1995 through December 2008.

if [ $yyyy$mm -lt 199505 ]; then
    sat="F00"
    ssmi_flag=0
    ssmis_flag=0
elif [ $yyyy$mm -le 200812 ]; then
    sat="F13"
    ssmi_flag=1
    ssmis_flag=0
else 
    sat="F17"
    ssmi_flag=0
    ssmis_flag=1
fi





#############################################
# Processing Step 1. (monthly)

#    a. Get listing of files for each technique
if [ $flag_GPCPcdr_d1_GetInputFiles -eq 1 ]; then
    echo "GPCPcdr_d1_GetInputFiles $yyyy $mm $sat" 
    GPCPcdr_d1/GPCPcdr_d1_GetInputFiles.sh $yyyy $mm $sat
fi



#    b. Check input files exist for processing 
GPCPcdr_d1/GPCPcdr_d1_ChkL1files.sh $yyyy $mm






#############################################
# get all interim files
# includes Algorithm processing and precip file reformatting

# GPROF processing
if [ $flag_GPCPcdr_d2_GPROFv2004 -eq 1 ]; then
    if [ ! -d $GPROFPATH/$yy$mm ]; then
	mkdir $GPROFPATH/$yy$mm
    fi

    # Process SSMI code
    if [ $ssmi_flag -eq 1 ]; then
	echo "GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMI/ $yyyy $mm $sat" 
	GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMI/GPCPcdr_d2_GPROFv2004_SSMI $yyyy $mm $sat ${GPROFPATH}/$yy$mm/

    # Process SSMIS code
    elif [ $ssmis_flag -eq 1 ]; then
	echo "GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMIS/ $yyyy $mm $sat"
	GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMIS/GPCPcdr_d2_GPROFv2004_SSMIS $yyyy $mm $sat ${GPROFPATH}/$yy$mm/
    fi
fi





#############################################
# Combination step

# Corr_G
# calculates corrections for IR data
if [ $flag_GPCPcdr_d4_CORR_G -eq 1 ]; then
    echo "GPCPcdr_d4_CORR_G $yyyy $mm"

    # create output file, if needed
    if [ ! -d $InterimPATH/CORR_G/$yyyy$mm ]; then
	mkdir $InterimPATH/CORR_G/$yyyy$mm
    fi

    # get list of GPROF files
    ls -1 ${InterimPATH}/GPROF2004V/${yy}${mm}/GPROF2004V* > file_listings/GPROFV4_1DD_${yyyy}${mm}.list

    # call code
    GPCPcdr_d4/GPCPcdr_d4_CORR_G/Main_GPCPcdr_d4_corr_g ${yyyy}${mm} ${InterimPATH} ${InputPATH}mergIR_3hr_1deg/ file_listings/mergIR_3hr_1deg_${yyyy}${mm}.list ${InterimPATH}/GPROF2004V/${yy}${mm}/ file_listings/GPROFV4_1DD_${yyyy}${mm}.list

fi


# ADJIR
# Applies corrections from CORR_G to IR data
if [ $flag_GPCPcdr_d4_ADJIR -eq 1 ]; then
    echo "GPCPcdr_d4_ADJIR $yyyy $mm"

    # create output file, if needed
    if [ ! -d $InterimPATH/ADJIR/$yyyy$mm ]; then
	mkdir $InterimPATH/ADJIR/$yyyy$mm
    fi

    # call code
    GPCPcdr_d4/GPCPcdr_d4_ADJIR/Main_GPCPcdr_d4_adjir ${yyyy}${mm} ${InterimPATH} ${InputPATH}mergIR_3hr_1deg/ file_listings/mergIR_3hr_1deg_${yyyy}${mm}.list 
fi


# CORR_T                                                                      
# calculates corrections for TOVS data
if [ $flag_GPCPcdr_d4_CORR_T -eq 1 ]; then
    echo "GPCPcdr_d4_CORR_T $yyyy $mm"

    # create output file, if needed
    if [ ! -d $InterimPATH/CORR_T/$yyyy$mm ]; then
	mkdir $InterimPATH/CORR_T/$yyyy$mm
    fi

    # call code
    GPCPcdr_d4/GPCPcdr_d4_CORR_T/Main_GPCPcdr_d4_corr_t ${yyyy}${mm} ${InterimPATH} ${OutputPATH} file_listings/TOVS_1DD_${yyyy}${mm}.list
fi


# FUSE2
# Merges all data to produce 1DD merged product
if [ $flag_GPCPcdr_d4_FUSE2 -eq 1 ]; then
    echo "GPCPcdr_d4_FUSE2 $yyyy $mm"

    # create output file, if needed
    if [ ! -d $OutputPATH/bin/$yyyy$mm ]; then
	mkdir $OutputPATH/bin/$yyyy$mm
    fi

    # Loop over days in month
    ndays=`date -d "${yyyy}-${mm}-01 +1 month - 1 day" +%d`
    for dd in `seq -w 1 $ndays`
    do
       # call code
       echo "processing ${yyyy}-${mm}-${dd}"
       GPCPcdr_d4/GPCPcdr_d4_FUSE2/Main_GPCPcdr_d4_FUSE2 ${yyyy}${mm}${dd} ${InterimPATH} ${OutputPATH} 
    done

fi


if [ $flag_GPCPcdr_d4_WriteNC -gt 0 ]; then
    echo GPCPcdr_d5_WriteNC $yyyy $mm 
    
    # create output file, if needed
    if [ ! -d $OutputPATH/$yyyy$mm ]; then
        mkdir $OutputPATH/$yyyy$mm
    fi
    
    # Write NetCDF4 output file
    # disable version check on CICS2 - avoids weird problem where
    # HDF5 header was different version to library (cause unknown)
    HDF5_DISABLE_VERSION_CHECK=2 GPCPcdr_d5/GPCPcdr_d5_WriteNC/GPCPcdr_d5_WriteNC $yyyy $mm v13rA1
    HDF5_DISABLE_VERSION_CHECK=2 GPCPcdr_d5/GPCPcdr_d5_WriteNC/GPCPcdr_d5_WriteRevNC $yyyy $mm v01r03

    # Would prefer to use compression in NetCDF4 file, but seems that this                         
    # requires users to install the correct compression utility                                    
    # For now, just gzip the NetCDF files (groan)
    gzip -f $OutputPATH/$yyyy$mm/*.nc

fi
