#/bin/sh
#@***h* GPCPcdr_d1_ChkL1files.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   GPCPcdr_d1_ChkL1files.sh
#
# PURPOSE
#   This code is to check input data exist and print summaries
#
# DESCRIPTION
#   Run from CODEPATH:
#   Chk_GPCPcdr_1month.sh 2013 02 
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 1/2017 Initial Version prepared for NCEI
#
#@*****

yyyy=$1
mm=$2


# check dates are valid
if [ $yyyy$mm -lt 199610 ]
then
    echo "Invalid year - GPCP 1DD starts in 1996/10. Exiting..."
    exit
fi
if [  $mm -lt 1 -o $mm -gt 12 ]
then
    echo "Month must be between 1 and 12. Exiting..."
    exit
fi


# print header
echo ""
echo "############################################"
echo " Printing summary of 1DD datasets for $yyyy/$mm"

# get file directories
InputPATH=$(echo `grep "InputPATH" OptionsD.h` | awk -F\"  '{print $2}')
InterimPATH=$(echo `grep "InterimPATH" OptionsD.h` | awk -F\"  '{print $2}')
GPROFPATH=$(echo `grep "GPROFPATH" OptionsD.h` | awk -F\"  '{print $2}')
MonthlyPATH=$(echo `grep "OutputPATH" ../monthly/OptionsM.h` | awk -F\"  '{print $2}')

# get 2-digit year
yy=`echo $yyyy | awk '{printf "%2.2d",substr($yyyy,3,2)}'`


# get number of days in month
if [  $mm -eq 1 -o $mm -eq 3 -o $mm -eq 5 -o $mm -eq 7 -o $mm -eq 8 -o $mm -eq 10 -o $mm -eq 12 ]
then
  DaysInMon=31
elif [ $mm -eq 4 -o $mm -eq 6 -o $mm -eq 9 -o $mm -eq 11 ]
then
  DaysInMon=30
elif [ $mm -eq 2 ]
then
  if [ `expr $yyyy % 4` -eq 0 ]
  then
    DaysInMon=29
  else
    DaysInMon=28
  fi
fi



###########################
# Check RSS SSMI/SSMIS data
NssmiFiles=`expr $DaysInMon \* 14`

# F17
if [ $yyyy$mm -ge 200901 ]; then
    echo ""
    if [ ! -d "${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/F17/${yyyy}/${mm}" ]; then
        echo "RSS SSMIS:    Directory does not exist: ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/F17/${yyyy}/${mm}/"
    else
        lines=`ls -1shSr ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/F17/${yyyy}/${mm}/`
        nlines=`echo "$lines" | wc -l`
        nlines=`expr $nlines - 1`
        min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "RSS SSMIS:    $nlines (expect ~ $NssmiFiles files); min: $min, max: $max"
    fi

# F13
elif [ $yyyy$mm -ge 199506 ]; then
    echo ""
    if [ ! -d "${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F13/${yyyy}/${mm}" ]; then
        echo "RSS SSMI F13:  Directory does not exist: ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F13/${yyyy}/${mm}/"
    else
        lines=`ls -1shSr ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F13/${yyyy}/${mm}/`
        nlines=`echo "$lines" | wc -l`
        nlines=`expr $nlines - 1`
        min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "RSS SSMI F13: $nlines (expect ~ $NssmiFiles files); min: $min, max: $max"
    fi
fi



###########################
# Check AIRS/TOVS 1DD data

# TOVS Data
if [ $yyyy$mm -ge 199610 ] && [ $yyyy$mm -le 200212 ]
then
    echo ""

    if [ ! -d "${InputPATH}/TOVS_1DD/${yyyy}/" ]; then
        echo "TOVS 1DD:    Directory does not exist: ${InputPATH}/TOVS_1DD/${yyyy}"
    else
        lines=`ls -lSr ${InputPATH}/TOVS_1DD/${yyyy}/i${yyyy}${mm}*.nobytes`
        nlines=`echo "$lines" | wc -l`
        min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $5}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $5}'`
        echo "TOVS 1DD:    $nlines / $DaysInMon files; min: $min, max: $max (expect: 259200)"
    fi
fi

# V6 Data
# this data is created by the monthly code and it will spit out empty files
# on days where there is no AIRS data.  Therefore, count non-empty files
# this requires the use of find with ls used as an -exec within find
if [ $yyyy$mm -ge 200301 ] 
then
    echo ""
    if [ ! -d "${InterimPATH}/AIRS_V6_1DD/${yyyy}/${mm}" ]; then
        echo "AIRS 1DD V6:  Directory does not exist: ${InterimPATH}/AIRS_V6_1DD/${yyyy}/${mm}/"
    else
	#lines=`ls -lSr ${InterimPATH}/AIRS_V6_1DD/${yyyy}/${mm}/*.avg`
        lines=`find "${InterimPATH}/AIRS_V6_1DD/${yyyy}/${mm}/" -maxdepth 1 -size +0 -type f -name "*.avg" -exec ls -lSr {} \;`
	nlines=`echo "$lines" | wc -l`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $5}'`
	max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $5}'`
	echo "AIRS V6:      $nlines / $DaysInMon files; min: $min, max: $max (expect: 259200)"
    fi
fi



################################
# Check 3hrly data IR data
if [ $yyyy -ge 1996 ]
then
    echo ""
    if [ ! -d "${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}" ]; then
	echo "3hrly IR:     Directory does not exist: ${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}"
    else
	lines=`ls -1shSr ${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}/hr*`
	nlines=`echo "$lines" | wc -l`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
	max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "3hrly IR:     $nlines / $DaysInMon files; min: $min, max: $max"
    fi
fi




######################
# GPCP Monthly Satellite/Gauge data
echo ""

# GPCP SG:
if [ -f "${MonthlyPATH}/bin/${yyyy}/${mm}/oc.${yyyy}${mm}.sg" ]
then
    fsize=`du -b "${MonthlyPATH}/bin/${yyyy}/${mm}/oc.${yyyy}${mm}.sg" | awk -F" "  '{print $1}'`
    echo "GPCP Monthly: File size: $fsize / 41472"
else
    echo "GPCP Monthly: File does not exist for $yyyy/$mm"
fi


# Print footer
echo ""
echo " End of 1DD Summary"
echo "############################################"
