      SUBROUTINE BILIN ( p_mon, LN1,  LT1,   x0, y0, LN2,  LT2, 
     +     xfac,  yfac, x_cyc, y_avg,  pcum, iret )
C************************************************************************
C* BILIN  Zoom grid with bilinear interpolation
C*
C*
C*
C* This subroutine zooms an image using bilinear interpolation.  If the
C*
C* output grid is a subset of the input grid, only the subset is
C*
C* computed, but using the whole input grid.  The location of the
C*
C* subset is user-specified.  In 1-D:
C*
C*
C*
C* Input grid:      |--------X--------|--------X- . . .
C*
C* Output grid:           |---X---|---X---|---X-- . . .
C*
C* In input coord.: |-x0->|
C*
C*
C*
C* The boundaries of the input grid are assumed cyclic on the east and
C*
C* west, and average on the north and south.  NOTE: This scheme makes
C*
C* the most sense for magnification (xfac,yfac > 1).
C*
C*
C*
C* BILIN  ( p_mon, LN1,  LT1, x0, y0, LN2, LT2, xfac, yfac, x_cyc,
C*
C*          y_avg, pcum, iret )
C*
C*
C*
C* Input parameters:
C*
C*   p_mon      REAL*   Coarse-grid input data set (size LN1,LT1)
C*
C*   LN1        INT     X size of p_mon
C*
C*   LT1        INT     Y size of p_mon
C*
C*   x0         REAL    Location of output grid's left edge in input
C*
C*                      grid coordinates (0=left)
C*
C*   y0         REAL    Location of output grid's top edge in input
C*
C*                      grid coordinates (0=top)
C*
C*   LN2        INT     X size of pcum
C*
C*   LT2        INT     Y size of pcum
C*
C*   xfac       REAL    X zoom factor
C*
C*   yfac       REAL    Y zoom factor
C*
C*   x_cyc      LOG     T|F=left,right boundary cyclic|missing
C*
c*   y_avg      LOG     T|F=top,bottom boundary avg-of-row|missing
*
C*
C*
C* Output parameters:
C*
C*   pcum       REAL*   Fine-grid input data set (size LN2,LT2)
C*
C*   iret       INT     Return code: 0 = normal
C*
C*                                  -1 = zero zoom factor
C*
C*                                  -2 = indexed beyond grid edge
C*
C**
C*
C* Log:
C*
C* G.Huffman/SSAI        8/98
C*
C* G.Huffman/SSAI        9/00   User-input boundaries; mask data at 50%
C*
C* G.Huffman/SSAI        2/04   Size decl.; decl. indices before
Carr.dec*
C************************************************************************
      IMPLICIT        NONE
C     
      REAL*4          sum, p1n, p1s, x1, dx1, y1, dy1, aa, ba, ab, bb
      INTEGER*4       n, i, i2, j2, i1a, i1b, j1a, j1b
      LOGICAL*4       use
C     
      REAL*4          RMISS
      PARAMETER       ( RMISS = -99999. )
C     
      INTEGER*4       LN1,   LT1,  LN2,  LT2, iret
      REAL*4          p_mon (LN1, LT1),  pcum (LN2, LT2),
     +     x0,    y0,   xfac, yfac
      LOGICAL*4       x_cyc, y_avg
C     
      iret = 0
C     
C     If the user requests it, compute the averages of the top and 
C     bottom rows for use in the boundary conditions there.
C     
      IF  ( y_avg )  THEN
         n   = 0
         sum = 0.
         DO  i = 1, LN1
            IF  ( p_mon (i, 1) .GE. 0. )  THEN
               n   = n   + 1
               sum = sum + p_mon (i, 1)
            END IF
         END DO
         IF  ( n .GT. 0 )  THEN
            p1n = sum / n
         ELSE
            p1n = RMISS
         END IF
         n   = 0
         sum = 0.
         DO  i = 1, LN1
            IF  ( p_mon (i, LT1) .GE. 0. )  THEN
               n   = n   + 1
               sum = sum + p_mon (i, LT1)
            END IF
         END DO
         IF  ( n .GT. 0 )  THEN
            p1s = sum / n
         ELSE
            p1s = RMISS
         END IF
C     
C     Otherwise, the user wants "missing" at the top and bottom.
C     
      ELSE
         p1n = RMISS
         p1s = RMISS
      END IF
C     
C     The concept is to express the output grid center location in
C     input grid coordinates.  In 1-D:
C     
C     Input grid:       |--------1--------|--------2- . . .
C     :                 :
C     Output grid:            |---1---|---2---|---3-- . . .
C     :                 :
C     In input coord.:  |-x0->|  :                 :
C     In output coord.:       |-->| (x2 = i2-.5)   :
C     :                 :
C     In input coord.:  |-------->| (x1 = x0 + x2/xfac)
C     :                 :
C     In input coord.:          i1a               i1b
C     (= INT[x1+.5])    (= i1a+1)
C     
C     Begin by checking that the zoom factors aren't zero and that
C     the first input grid location requested isn't before the 
C     boundary condition locations.
C     
      IF  ( xfac .EQ. 0. .OR. yfac .EQ. 0. )  THEN
         WRITE (*, *) 'Error: Zero value in xfac, yfac =', xfac, yfac
         iret = -1
         RETURN
      END IF
      IF  ( x0 + .5 / xfac + .5 .LT. 0. )  THEN
         WRITE (*, *) 'Error: Reaching before grid to i1a =',
     +        x0 + .5 / xfac + .5
         WRITE (*, *) 'with x0, xfac =', x0, xfac
         iret = -2
         RETURN
      END IF
      IF  ( y0 + .5 / yfac + .5 .LT. 0. )  THEN
         WRITE (*, *) 'Error: Reaching before grid to j1a =',
     +        y0 + .5 / yfac + .5
         WRITE (*, *) 'with y0, yfac =', y0, yfac
         iret = -2
         RETURN
      END IF
C     
C     Set up indices, and check that the last input grid location
C     requested isn't past the boundary condition locations.
C     
      DO  j2 = 1, LT2
         y1  = y0 + ( j2 - .5 ) / yfac + .5
         dy1 = AMOD ( y1, 1. )
         j1a = INT  ( y1 )
         j1b = j1a + 1
         IF  ( j1b .GT. LT1 + 1 )  THEN
            WRITE (*, *) 'Error: Reaching past grid to j1b =', 
     +           j1b, 'with j2, y0, yfac =', j2, y0, yfac
            iret = -2
            RETURN
         END IF
         DO  i2 = 1, LN2
            x1  = x0 + ( i2 - .5 ) / xfac + .5
            dx1 = AMOD ( x1, 1. )
            i1a = INT  ( x1 )
            i1b = i1a + 1
            IF  ( i1b .GT. LN1 + 1 )  THEN
               WRITE (*, *) 'Error: Reaching past grid to i1b =', 
     +              i1b, 'with i2, x0, xfac =', i2, x0, 
     +              xfac
               iret = -2
               RETURN
            END IF
C     
C           Handle boundary conditions, first cyclic east and west
C           (it's just indexing), then average north and south 
C           (which actually involves setting values).  The template
C           is:
C              aa ba
C              ab bb
C
            IF  ( i1a .EQ. 0       )  i1a = LN1
            IF  ( i1b .EQ. LN1 + 1 )  i1b = 1
            IF  ( j1a .EQ. 0       )  THEN
               aa = p1n
               ba = p1n
            ELSE
               aa = p_mon (i1a, j1a)
               ba = p_mon (i1b, j1a)
            END IF
            IF  ( j1b .EQ. LT1 + 1 )  THEN
               ab = p1s
               bb = p1s
            ELSE
               ab = p_mon (i1a, j1b)
               bb = p_mon (i1b, j1b)
            END IF
C     
C           Now re-visit values if the east-west boundary condition
C           is missing instead of cyclic.  [The missing top/bottom
C           was set in pln, pls above.]
C
            IF  ( .NOT. x_cyc )  THEN
               IF  ( i1a .EQ. 0 )  THEN
                  aa = RMISS
                  ab = RMISS
               ELSE IF  ( i1b .EQ. LN1 + 1 )  THEN
                  ba = RMISS
                  bb = RMISS
               END IF
            END IF
C     
C           Cope with missing data.  There must be at least 1 
C           value out of the 4, in which case fill missings with 
C           the average of the available data.  Only points within
C           the original low-res. grid boxes are given high-res.
C           values, except box centers right on the boundary are
C           also allowed.  This grows the data area slightly.
C
            n   = 0
            sum = 0.
            use = .FALSE.
            IF  ( aa .GE. 0. )  THEN
               n   = n + 1
               sum = sum + aa
               IF  ( dx1 .LE. .5 .AND. dy1 .LE. .5 )  use = .TRUE.
            END IF
            IF  ( ba .GE. 0. )  THEN
               n   = n + 1
               sum = sum + ba
               IF  ( dx1 .GE. .5 .AND. dy1 .LE. .5 )  use = .TRUE.
            END IF
            IF  ( ab .GE. 0. )  THEN
               n   = n + 1
               sum = sum + ab
               IF  ( dx1 .LE. .5 .AND. dy1 .GE. .5 )  use = .TRUE.
            END IF
            IF  ( bb .GE. 0. )  THEN
               n   = n + 1
               sum = sum + bb
               IF  ( dx1 .GE. .5 .AND. dy1 .GE. .5 )  use = .TRUE.
            END IF
            IF  ( n .LT. 1 .OR. .NOT. use )  THEN
               pcum (i2, j2) = RMISS
            ELSE
               sum = sum / n
               IF  ( aa .LT. 0. )  aa = sum
               IF  ( ba .LT. 0. )  ba = sum
               IF  ( ab .LT. 0. )  ab = sum
               IF  ( bb .LT. 0. )  bb = sum
C     
C              If any of the tests set use to .TRUE. and there's 
C              at least one point, do the interpolation; note the 
C              symmetry of the formula.
C
               pcum (i2, j2) = ( bb - ab - ba + aa ) * dx1 * dy1 +
     +              ( ba - aa ) * dx1 + 
     +              ( ab - aa ) * dy1 +
     +              aa
            END IF
         END DO
      END DO
C     
      RETURN
      END
      
