      SUBROUTINE PBCORR  ( datin, histo, histl, NSKPLT, LT, datout, 
     +     iret )
C***********************************************************************
C* PBCORR  Adjust rain data using land/ocean cum. histograms           *
C*                                                                     *
C* This subroutine uses a previously computed PDF matching between the *
C* SSM/I and TMI cumulative histograms (land and ocean separately) to  *
C* compute a correction to the SSM/I data.  If land, ocean, or both    *
C* cumulative correction histograms are missing, ocean, land, or no    *
C* correction is performed everywhere.  Any rows that are skipped in   *
C* processing are RMISS-filled on output (unless there is no           *
C* correction).                                                        *
C* ==>> Note: datin and datout are transposed from the usual GPCP <<== *
C* ==>> orientation; they are 360x720.                            <<== *
C*                                                                     *
C* PBCORR  ( datin, histo, histl, LTSKP, LT, datout, iret )            *
C*                                                                     *
C* Input Parameters:                                                   *
C*  datin       REAL*   Input SSM/I half-orbit average data (size      *
C*                      MXLN,MXLT)                                     *
C*  histo       REAL*   Histogram of ocean values (size MXBIN,2);      *
C*                      histo (1,1) < 0 flags no ocean histogram data  *
C*  histl       REAL*   Histogram of land values (size MXBIN,2)        *
C*                      histl (1,1) < 0 flags no land histogram data   *
C*  NSKPLT      INT     Number of .5-deg latitude columns to skip      *
C*                      before beginning processing                    *
C*  LT          INT     Number of .5-deg latitude columns to process   *
C*                                                                     *
C* Output Parameters:                                                  *
C*  datout      REAL*   Adjusted SSM/I half-orbit average data (size   *
C*                      MXLN,MXLT)                                     *
C*  iret        INT     Return code: 0 = normal                        *
C*                                  -1 = stop                          *
C**                                                                    *
C* Log:                                                                *
C* D.Bolvin/SSAI         5/00   adjssmi.F                              *
C* G.Huffman/SSAI        5/00   Error codes, split input/output array, *
C*                              only read sfc file once; missing hist. *
C* D.Bolvin/SSAI         3/08   CHARACTER*80 -> CHARACTER*256          *
C***********************************************************************
C
      IMPLICIT        NONE
      INTEGER*4       MXLN,  MXLT, MXBIN
      REAL*4          AMASK, RMISS
      PARAMETER       ( MXLN  = 720,
     +     MXLT  = 360,
     +     MXBIN = 10000,
     +     AMASK = 70.,
     +     RMISS = -99999. )
C     
      INTEGER*4       i, j,   k,  nlowlim,   nhighlim, iunit,  nin,
     +     NSKPLT, LT, iret,      jstrt,    jstop
C     
      REAL*4          ssmip, ssmic, slope, aint, sfcthr
      REAL*4          datin (MXLT,  MXLN), datout (MXLT,  MXLN), 
     +     histo (MXBIN, 2),    histl (10000, 2), 
     +     sfc   (MXLN,  MXLT)
      CHARACTER*256   fnam
      DATA            fnam  / 'surfrac.0.5' /
C     
      iret = 0


C***********************************************************************
C       If it's the first time, read the percent surface water coverage 
C       file.  After that the data are SAVE'd.


      CALL Read_Binary_File (fnam, MXLN, MXLT, 1, sfc)


C     
C***********************************************************************
C       Set the latitude index limits of the actual conversion zone.
C
      jstrt = NSKPLT + 1
      jstop = NSKPLT + LT
      IF  ( jstop .GT. MXLT )  THEN
         WRITE (*, *) 'Warning: NSKPLT, LT =', NSKPLT, LT, 
     +        ' runs past MXLT =', MXLT
         jstop = MXLT
      END IF
      IF  ( jstrt .GT. jstop )  THEN
         WRITE (*, *) 'Error: jstrt =', jstrt, ' .GT. jstop =',
     +        jstop
         iret = -1
         RETURN
      END IF
C     
C     Check for no histograms at all (just copy data to output), no 
C     ocean (set surface threshold to always do land), or no land (set
C     sfc threshold to always do ocean).
C
      IF  ( histo (1, 1) .LT. 0. )  THEN
         IF  ( histl (1, 1) .LT. 0. )  THEN
            DO  i = 1, MXLN
               DO  j = 1, MXLT
                  datout (j, i) = datin (j, i)
               END DO
            END DO
            RETURN
         ELSE
            sfcthr = 101.
         END IF
      ELSE IF  ( histl (1, 1) .LT. 0. )  THEN
         sfcthr = -1.
      ELSE
         sfcthr = AMASK
      END IF
      
C     Adjust the SSM/I data by matching the cumulative histogram with
C     the TMI data.  Start by blanking out the regions that are
C     skipped.
C     
      IF  ( jstrt .GT. 1 )  THEN
         DO  i = 1, MXLN
            DO  j = 1, jstrt - 1
               datout (j, i) = RMISS
            END DO
         END DO
      END IF
      IF  ( jstop .LT. MXLT )  THEN
         DO  i = 1,         MXLN
            DO  j = jstop + 1, MXLT
               datout (j, i) = RMISS
            END DO
         END DO
      END IF
C     
      DO  i = 1,     MXLN
         DO  j = jstrt, jstop
            IF  ( datin (j, i) .GT. 100. )  THEN
               WRITE (*,*) 'Error: SSM/I = ', datin (j, i),
     +              ' .GT. 100 mm/hr; stopping in pbcorr'
               iret = -1
               RETURN
            ELSE IF  ( datin (j, i) .LE. 0. )  THEN
               GO TO 10
            ELSE
               nlowlim  = INT (datin (j, i) * 100) + 1
               nhighlim = nlowlim + 1
            END IF
C     
            IF  ( sfc (i, j) .GE. sfcthr )  THEN
C     
               slope = histo (nhighlim, 1) - histo (nlowlim, 1)
               aint  = histo (nlowlim,  1) - (slope * FLOAT (nlowlim))
               ssmic = (slope * ((datin (j, i) * 100.0) + 1.0)) + aint
C     
               DO  k = 2, MXBIN
                  IF  ( ssmic .LT. histo (k, 2) )  THEN
                     slope = 1. / (histo (k, 2) - histo (k-1, 2))
                     aint  = FLOAT (k - 1) -  
     +                    (slope * histo(k - 1, 2))
                     datout (j, i) = 
     +                    (((slope * ssmic) + aint) - 1. ) / 100.
                     GO TO 10
                  END IF
               END DO
C     
            ELSE
               slope = histl (nhighlim, 1) - histl (nlowlim, 1)
               aint  = histl (nlowlim,  1) - 
     +              (slope * FLOAT (nlowlim))
               ssmic = (slope * ((datin (j, i) * 100.) + 1.)) + aint
C     
               DO  k = 2, MXBIN
                  IF  ( ssmic .LT. histl (k, 2) )  THEN
                     slope = 1. / (histl (k, 2) - histl (k-1, 2))
                     aint  = FLOAT (k - 1) - (slope * histl (k-1, 2))
                     datout (j, i) = 
     +                    (((slope * ssmic) + aint) - 1. ) / 100.
                     GO TO 10
                  END IF
               END DO
C     
            END IF
C     
 10         CONTINUE
C     
         END DO
      END DO
C     
      RETURN
      END
      
      
