      SUBROUTINE SMHIST4 ( nin, NLAY, LN, LT, MINSM, nout, iret )
C************************************************************************
C* SMHIST4  Spatially smooth the I*4 high-res IR histogram array
C*
C*
C*
C* This subroutine smooths the I*4 high-resolution IR histogram array
C*
C* on a 3x3 boxcar.  At least MINSM values must contribute for a point
C*
C* to be non-missing.  "Missing" is flagged by a negative number.  The
C*
C* code is written to give correct behavior when the output and input
C*
C* arrays point to the same array in the calling program.
C*
C*
C*
C* Note: This code could be generalized by extending LNMN,LNMX further
C*
C* out and lengthening nhold; the test on MINSM would be variable, a
C*
C* variable number of lines would get loaded the first time, a variable
C*
C* number would be loaded missing toward the end, and a variable
C*
C* number of lateral boundary points would be loaded.
C*
C*
C*
C* SMHIST4  ( nin, LN, LT, MINSM, nout, iret )
C*
C*
C*
C* Input parameters:
C*
C*   nin        INT*    Input array (size NLAY,LN,LT)
C*
C*   NLAY       INT     Number of layers
C*
C*   LN         INT     X size of nin, nout
C*
C*   LT         INT     Y size of nin, nout
C*
C*   MINSM      INT     Minimum number of points in template to get a
C*
C*                      non-missing smoothed value
C*
C*
C*
C* Output parameters:
C*
C*   nout       INT*    Output array (size 189:270,LN,LT)
C*
C*   iret       INT     Return code: 0 = normal
C*
C*                                  -1 = LN .GT. max LN of work array
C*
C*                                  -2 = MINSM outside [1,9]
C*
C**
C*
C* Log:
C*
C* G.Huffman/SSAI        8/98   SMHIST
C*
C* G.Huffman/SSAI        7/00   SMHIST4; Make NLAY an input (orig.
C*
C*                              189:270)
C*
C* G.Huffman/SSAI        1/02   Boost MXLAY to 131
C*
C* G.Huffman/SSAI        2/04   Decl. indices before array decl.
C*
C************************************************************************
      IMPLICIT        NONE
C     
      INTEGER*4       MXLAY, LNMN, LNMX, I2MISS
      PARAMETER       ( MXLAY  = 131,
     +     LNMN   = 0,
     +     LNMX   = 361,
     +     I2MISS = -31999 )
C     
      INTEGER*4       nhold (MXLAY, LNMN:LNMX, 3), 
     +     it, i, j,  ii, jj, n, nsum
C     
      INTEGER*4       NLAY,  LN, LT, MINSM, iret
      INTEGER*4       nin (NLAY, LN, LT), nout (NLAY, LN, LT)
C     
      iret = 0
C     
C     Check that the NLAY and X dimensions are correct and that the 
C     minimum weight is within bounds.
C     
      IF  ( NLAY .GT. MXLAY )  THEN
         WRITE (*, *) 'Error: Input NLAY .GT. work array NLAY:',
     +        NLAY, MXLAY
         iret = -1
         RETURN
      END IF
      IF  ( LN .GT. LNMX-1 )  THEN
         WRITE (*, *) 'Error: Input LN .GT. work array LN:', LN, 
     +        LNMX-1
         iret = -1
         RETURN
      END IF
      IF  ( MINSM .LT. 1 .OR. MINSM .GT. 9 )  THEN
         WRITE (*, *) 'Error: Input MINSM =', MINSM, 
     +        ' outside [1,9]'
         iret = -2
         RETURN
      END IF
C     
C     General concept:  On each iteration, the hold array (dimensioned
C     hist.class,lon,3) has been loaded with all the input for a 
C     row.  The computation can be sent directly to the output array
C     even if it's the same as the input array in the call, because 
C     the "old" value being overwritten is already transferred to 
C     nhold.  Shift the hold array up by one layer, and fill the third 
C     layer with the next (input) row's layer.  Repeat.
C     
C     Startup:  Load all three layers of the hold array.  This 
C     involves setting the north boundary condition (missing) and the
C     east and west conditions (cyclic).
C     
      DO  it = 1, NLAY
         nhold (it, 0,    1) = I2MISS
         nhold (it, 0,    2) = nin (it, LN, 1)
         nhold (it, 0,    3) = nin (it, LN, 2)
      END DO
      DO  i  = 1,   LN
         DO  it = 1, NLAY
            nhold (it, i,    1) = I2MISS
            nhold (it, i,    2) = nin (it, i,  1)
            nhold (it, i,    3) = nin (it, i,  2)
         END DO
      END DO
      DO  it = 1, NLAY
         nhold (it, LN+1, 1) = I2MISS
         nhold (it, LN+1, 2) = nin (it, 1,  1)
         nhold (it, LN+1, 3) = nin (it, 1,  2)
      END DO
C     
C     Main loop implementing the general concept (above).  Compute
C     the row's result into the second layer of nres.
C     
      DO  j = 1, LT
         DO  i  = 1,   LN
            DO  it = 1, NLAY
               n    = 0
               nsum = 0
               DO  jj = 1,   3
                  DO  ii = i-1, i+1
                     IF  ( nhold (it, ii, jj) .GE. 0 )  THEN
                        n    = n    + 1
                        nsum = nsum + nhold (it, ii, jj)
                     END IF
                  END DO
               END DO
               IF  ( n .LT. MINSM )  THEN
                  nout (it, i, j) = I2MISS
               ELSE
                  nout (it, i, j) = NINT ( FLOAT ( nsum ) /
     +                 FLOAT ( n    ) )
               END IF
            END DO
         END DO
C     
C     The last time no further work is needed.  Otherwise, shift
C     nhold up one row and load up the 3d row, accounting
C     for the west and east boundaries (cyclic).  The next-to-
C     the-last time (readying for the last computation) handles 
C     the south boundary (missing) as well.
C     
         IF  ( j .NE. LT )  THEN
            DO  i  = 0,   LN+1
               DO  it = 1, NLAY
                  nhold (it, i, 1) = nhold (it, i, 2)
                  nhold (it, i, 2) = nhold (it, i, 3)
               END DO
            END DO
C     
            IF  ( j .NE. LT-1 )  THEN
               DO  it = 1, NLAY
                  nhold (it, 0,    3) = nin (it, LN, j+2)
               END DO
               DO  i  = 1,   LN
                  DO  it = 1, NLAY
                     nhold (it, i,    3) = nin (it, i,  j+2)
                  END DO
               END DO
               DO  it = 1, NLAY
                  nhold (it, LN+1, 3) = nin (it, 1,  j+2)
               END DO
            ELSE
               DO  it = 1, NLAY
                  nhold (it, 0,    3) = I2MISS
               END DO
               DO  i  = 1,   LN
                  DO  it = 1, NLAY
                     nhold (it, i,    3) = I2MISS
                  END DO
               END DO
               DO  it = 1, NLAY
                  nhold (it, LN+1, 3) = I2MISS
               END DO
            END IF
         END IF
      END DO    
C     
      RETURN
      END
      
      
