      SUBROUTINE SMOOTH  ( ismth,  smth, wtmin, arrin, LN, LT, 
     +     arrout, iret )
C************************************************************************
C* SMOOTH  Smooth a LNxLT field
C*
C*
C*
C* Smooth a LNxLT field.  The north and south boundaries are filled
C*
C* with "missing" (-99999.), while the east and west are periodic.  The
C*
C* smoother must be square, and missing values simply give relatively
C*
C* more weight to the good values contributing to a point (unless there
C*
C* is less than wtmin weight).
C*
C*
C*
C* SMOOTH  ( ismth, smth, wtmin, arrin, LN, LT, arrout, iret )
C*
C*
C*
C* Input parameters:
C*
C*   ismth      INT     Dimension of smoothing box (in pixels)
C*
C*   smth       REAL*   Smoothing array (size MXSM,MXSM)
C*
C*   wtmin      REAL    Minimum weight for a point to be non-missing
C*
C*   arrin      REAL*   Input data (size LN,LT)
C*
C*   LN         INT     X size
C*
C*   LT         INT     Y size
C*
C*
C*
C* Output parameters:
C*
C*   arrout     REAL*   Output smoothed data (size MXLNCL,MXLTCL)
C*
C*   iret       INT     Return code: 0 = normal
C*
C*                                  -1 = ismth out of bounds or even
C*
C*                                  -2 = no data in input array at all
C*
C**
C*
C* Log:
C*
C* G.Huffman/USRA        3/92
C*
C* G.Huffman/USRA       10/92   Change to passing in dimensions
C*
C* G.Huffman/USRA        2/93   Add wtmin
C*
C* G.Huffman/SSAI        3/00   Check for "no data in input"; write msg
C*
C*                              on return error -1
C*
C* G.Huffman/SSAI        2/04   Size dec.; bigger MXX, MXY; decl.
C*
C*                              indices before array decl.
C*
C************************************************************************
      IMPLICIT        NONE
      INTEGER*4       i,  ii,  j, jj, k, is_2
      REAL*4          wt, xwt, reject
C     
C     Args are (ln,lt).
C     
      INTEGER*4       MXSM, MS_2, MXX, MXY
      REAL*4          RMISS
      PARAMETER       ( MXSM  = 15,
     +     MS_2  = MXSM / 2,
     +     MXX   = 1440,
     +     MXY   = 720,
     +     RMISS = -99999. )
C     
      INTEGER*4       ismth, LN, LT, iret
      REAL*4          wtmin,          smth   (MXSM, MXSM),
     +     arrin (LN, LT), arrout (LN,   LT)
C     
      REAL*4          aug   (MXX + 2 * MS_2, MXY + 2* MS_2),
     +     augwt (MXX + 2 * MS_2, MXY + 2* MS_2)
C     
C     Check that ismth is odd and in bounds.
C     
      IF  ( ismth .LT. 3 .OR. MOD ( ismth, 2 ) .NE. 1 .OR.
     +     ismth .GT. MXSM )  THEN
         WRITE (*, *) 'Error: smoothing routine found ismth (=', 
     +        ismth, ') <3, even, or >MXSM (=', MXSM, ').'
         iret = -1
         RETURN
      END IF
C     
C     If there's no data in the input array reflect the input array
C     into the output array with an error.
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( arrin (i, j) .NE. RMISS )  GO TO 10
         END DO
      END DO
      DO  j = 1, LT
         DO  i = 1, LN
            arrout (i, j) = arrin (i, j)
         END DO
      END DO
      WRITE (*, *) 'Error: smoothing routine found no data.'
      iret = -2
      RETURN
 10   CONTINUE
C     
C     Set the minimum weight limit.
C     
      IF  ( wtmin .LT. 0. )  THEN
         reject = 0.
      ELSE
         reject = wtmin
      END IF
C     
C     Load an augmented array of data so that indexing the convolution
C     is easy.  The "picture frame" is loaded all the way out to MS_2
C     (rather than is_2) to simplify indexing.  The fill is RMISS
C     (missing) above and below, and periodic left and right.
C     
      is_2 = ismth / 2
      DO  j = 1, LT
         DO  i = 1, LN
            aug (MS_2 + i, MS_2 + j) = arrin (i, j)
         END DO
      END DO
C     
      DO  j = 1, MS_2
         DO  i = 1, LN + 2 * MS_2
            aug (i, j            ) = RMISS
            aug (i, j + MS_2 + LT) = RMISS
         END DO
      END DO
C     
      DO  j = MS_2 + 1, MS_2 + LT
         DO  i = 1,        MS_2
            aug (i,             j) = aug (i + LN  , j)
            aug (i + LN + MS_2, j) = aug (i + MS_2, j)
         END DO
      END DO
C     
C     Do the convolution, indexing on aug corner and smth center.
C     
      DO  j = MS_2 + 1, MS_2 + LT
         DO  i = MS_2 + 1, MS_2 + LN
            wt  = 0.
            xwt = 0.
            DO  jj = -is_2, is_2
               DO  ii = -is_2, is_2
                  IF  ( aug (ii + i, jj + j) .NE. RMISS )  THEN
                     wt  = wt  + smth (ii + is_2 + 1, jj + is_2 + 1)
                     xwt = xwt + smth (ii + is_2 + 1, jj + is_2 + 1) *
     +                    aug  (ii + i,        jj + j)
                  END IF
               END DO
            END DO
            augwt (i, j) = wt
            IF  ( wt .LE. reject )  THEN
               arrout (i - MS_2, j - MS_2) = RMISS
            ELSE
               arrout (i - MS_2, j - MS_2) = xwt / wt
            END IF
         END DO
      END DO
C     
      iret = 0
      RETURN
      END
      
