C@***h* Main_GPCPcdr_d4_corr_t.f
C       Global Precipitation Climatology Project (GPCP)
C
C NAME
C   Main_GPCPcdr_d4_corr_t.f
C
C PURPOSE
C   To match TOVS to GPCP monthly
C
C DESCRIPTION
C  This program computes the correction coefficients for daily TOVS
C  from daily TMPI estimates and monthly GPCP SG fields.
C
C  Input data
C    TO_NAMES      CHAR    File names of daily merged TOVS data;
C                   - files are sorted in ascending date/time,
C                   - there can be up to31 entries,
C                   - with the last entry is '999';
C                   or the name of a file containing such a list
C                   (without the terminating '999') as NAME,file
C    TM_NAMES      CHAR    As above for corresponding TMPI files
C    MON_NAME      CHAR      File name of 2.5x2.5-deg monthly precip field
C                   (in mm/d) for getting rain rate, or 'none'
C    DATE          CHAR      Date as YYYYMM
C    OUTBASE       CHAR      Base name of output files
C    SMOOTH        CHAR      Size of spatial smoother for IR histograms and
C                   SSM/I fractional coverage; 0 = none, 3 = only
C                   valid non-zero value
C  
C AUTHOR
C   Robert Adler, Mathew Sapiano, Jian-Jian Wang
C   University of Maryland College Park
C
C COPYRIGHT
C   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE
C   PUBLIC
C   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
C   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
C   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
C   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
C   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
C   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE
C   TECHNICAL
C   SUPPORT TO USERS.
C
C HISTORY
C 1/2017 Initial Version prepared for NCEI
C
C@*****      
      program Main_GPCPcdr_d4_corr_g

      IMPLICIT      NONE
      
      integer LN,    LT, MAXNAM, MAXPDF
      real RMISS
      parameter (LN     = 360,
     +     LT     = 180,
     +     MAXNAM = 31,
     +     MAXPDF = MAXNAM * 9,
     +     RMISS  = -99999.)

      character*256, InterimPATH, OutputPATH, TOVSlist
      character*6 yyyymm
      character*2 dd
      integer Ninputs, inunit, ios, strlen, l_fbas, idx
      integer i, j, k, m, ii, jj, jdex, iw, iret, idex, ie
      integer TOVSflag
      logical opn, xst
      integer*4     iyear, imonth, ndays, npass, npts
      integer*4     NdaysInMonth(12)
     +     / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /
      
      CHARACTER*256 toname(MAXNAM), tmname, mname, fbas, fname, tmpname
            
      integer npdf(LN,LT), ntmpi(LN,LT), nto, nn, ns
      real redto(LN,LT), tmpif(LN,LT), ratf(LN,LT), tmpi(LN,LT), 
     +     pcum(LN,LT),
     +     tmpif40n(LN), tmpif40s(LN),  tovsf40n(LN), tovsf40s(LN),
     +     tovsfn, tovsfs, tmpifn, tmpifs, ratn, rats
      real tovs(LN,LT), tovsl(LN,LT), z_pt(LN,LT), rat(LN, LT)
      real pdf (MAXPDF, LN, LT), pdf3 (MAXPDF, LN, LT+3)
      real tmp_arr(360,80)
      real p_mon(144,72)
      
      INTEGER nsm
      
      REAL smth (11, 11)
      DATA smth / 121*1 /
      
      

      
c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 4 ) then
         WRITE(*,*) 'Error in GPCPcdr_d4_CORR_T:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif
      
C     Get arguments and set the length of the month in hours.
      call getarg(1, yyyymm)
      call getarg(2, InterimPATH)
      call getarg(3, OutputPATH)
      call getarg(4, TOVSlist)
      
c     Convert month to a number
      read(yyyymm(1:4), '(i10)' ) iyear
      read(yyyymm(5:6), '(i10)' ) imonth
      
c     Determine number of days in month
      ndays=NdaysInMonth(imonth)
      if ( imonth.EQ.2 ) then 
         if ( mod(iyear,400).EQ.0 ) then
            ndays=29
         elseif ( mod(iyear,100).EQ.0 ) then
            ndays=28
         elseif ( mod(iyear,4).EQ.0 ) then
            ndays=29
         endif
      endif
      
c     Get monthly precip field name and read into p_mon
      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      mname = InterimPATH(1:strlen)//'/FUSE2/oc.'//yyyymm//'.sg'
c     Read monthly CDR data from mname
      CALL Read_Binary_File (mname, 144, 72, 1, p_mon)
      
      
c     get basename for output files
      fbas = InterimPATH(1:strlen)//'/CORR_T/'//yyyymm//'/1dd.'//yyyymm
c     Get length of fbas
      do l_fbas = 1,len_trim(fbas)
         if (fbas(l_fbas:l_fbas) .eq. ' ') then
            continue
         end if
      end do
      l_fbas=l_fbas-1
      




c     set size of spatial smoother
      nsm=3
      
c     Read contents of TOVSlist
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      
      open(inunit, file=TOVSlist, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_t:'
     &        //'Failed to open TOVS file listing file'
         STOP
      end if
      nto=0
      do i=1,MAXNAM
         read(inunit, '(A256)', iostat=ios ) toname(i)
         if ( ios .EQ. 0 ) then
            nto=nto+1
         else
            toname(i) = '999'
         endif
      end do
      if ( nto .EQ. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_t:'
     &        //'Failed to read TOVS file listing file'
         STOP
      end if
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_t:'
     &        //'Failed to close TOVS file listing file'
         STOP
      end if
      
      
      


c************************************************************
C     Initialize fields.
C     
      DO  i = 1, LN
         tovsf40n (i) = 0.
         tovsf40s (i) = 0.
         tmpif40n (i) = 0.
         tmpif40s (i) = 0.
      END DO
      nn = 0
      ns = 0
      tovsfn = 0.
      tovsfs = 0.
      tmpifn = 0.
      tmpifs = 0.
      DO  j = 1, LT
         DO  i = 1, LN
            redto (i, j) = 0.
            tmpif (i, j) = 0.
            ntmpi (i, j) = 0
            ratf  (i, j) = RMISS
         END DO
      END DO
      IF  ( nto .LT. 31 )  THEN
         DO  j = 1, LT
            DO  i = 1, LN
               DO  m = nto+1, MAXNAM
                  pdf (m, i, j) = RMISS
               END DO
            END DO
         END DO
      END IF
      

C     General concept: Assuming that the user has matched days
C     of TOVS and TMPI, open one of each, accumulate the TOVS PDF,
C     the TMPI f, the matched TOVS and TMPI f over the 3 boxes
C     centered at 38.5N for all longitudes, and the same for 38.5S.
      
      DO  m = 1, nto

c        Read TOVS data and byte swap
         tmpname=toname(m)
         TOVSflag=0
         idx=index(tmpname,'.avg')
         if (idx .EQ. 0) then
            idx=index(tmpname,'.utime')
            TOVSflag=1
         end if
         dd=tmpname(idx-2:idx-1)

         CALL Read_Binary_File (toname(m), 360, 180, 1, tovs)
         if (TOVSflag .EQ. 1) CALL SWAPR4 (tovs, 360, 180)



c        Read TMPI from ADJIR
         do strlen = 1,len_trim(InterimPATH)
            if (InterimPATH(strlen:strlen) .eq. ' ') then
               continue
            end if
         end do
         strlen=strlen-1
         
         fname=InterimPATH(1:strlen)//'/ADJIR/'//yyyymm//'/1dd.'
     +        //yyyymm//dd//'.p'

         CALL Read_Binary_File (fname, 360, 80, 1, tmp_arr)
         DO  j = 1, 80
            DO  i = 1, LN
               tmpi (i, j+50) = tmp_arr (i, j)
            END DO
         END DO

         
c        Copy data into other arrays
         DO  j = 1, LT
            DO  i = 1, LN
               pdf (m, i, j) = tovs (i, j)
            END DO
         END DO
         DO  j = 51, LT-50
            DO  i = 1,  LN
               IF  ( tmpi (i, j) .GE. 0. ) THEN
                  ntmpi (i, j) = ntmpi (i, j) + 1
                  IF  ( tmpi (i, j) .GT. 0. )  
     +                 tmpif (i, j) = tmpif (i, j) + 1.
               END IF
            END DO
         END DO
C     
         DO  j = 51, 53
            DO  i = 1,  LN
               IF  ( tovs (i, j) .GE. 0. .AND. tmpi (i, j) .GE. 0. ) 
     +              THEN
                  nn = nn + 1
                  IF  ( tovs (i, j) .GT. 0. )  THEN
                     tovsf40n (i) = tovsf40n (i) + 1.
                     tovsfn       = tovsfn       + 1.
                  END IF
                  IF  ( tmpi (i, j) .GT. 0. )  THEN
                     tmpif40n (i) = tmpif40n (i) + 1.
                     tmpifn       = tmpifn       + 1.
                  END IF
               END IF
            END DO
         END DO
         
         DO  j = 128, 130
            DO  i = 1,   LN
               IF  ( tovs (i, j) .GE. 0. .AND. tmpi (i, j) .GE. 0. ) 
     +              THEN
                  ns = ns + 1
                  IF  ( tovs (i, j) .GT. 0. )  THEN
                     tovsf40s (i) = tovsf40s (i) + 1.
                     tovsfs       = tovsfs       + 1.
                  END IF
                  IF  ( tmpi (i, j) .GT. 0. )  THEN
                     tmpif40s (i) = tmpif40s (i) + 1.
                     tmpifs       = tmpifs       + 1.
                  END IF
               END IF
            END DO
         END DO
      END DO


C     Implement 3x3 boxcar smoothing on PDF and sort PDF into 
C     ascending order.
C     
      DO  j = 1, LT
         DO  i = 1, LN
            k = -1
            DO  jj = j-1, j+1
               IF  ( jj .EQ. 0 )  THEN
                  jdex = 1
               ELSE IF  ( jj .EQ. LT + 1 )  THEN
                  jdex = LT
               ELSE
                  jdex = jj
               END IF
               DO  ii = i-1, i+1
                  k = k + 1
                  IF  ( ii .EQ. 0 )  THEN
                     idex = LN
                  ELSE IF  ( ii .EQ. LN + 1 )  THEN
                     idex = 1
                  ELSE
                     idex = ii
                  END IF
                  DO  m = 1, nto
                     pdf3 (m + k*nto, i, j) = pdf (m, idex, jdex)
c                    Following line replaces an "EQUIVALENCE" statement
c                    that was used in original code
cc                     pdf3(1, 1, 4)=pdf(1, 1, 1)
                  END DO
               END DO
            END DO
         END DO
      END DO
      
      npts = nto * 9
      IF  ( npts .GT. MAXPDF )  THEN
         WRITE (*, *) 'Error: npts =', npts, ' > MAXPDF =', MAXPDF
         STOP
      END IF

      DO  j = 1, LT
         DO  i = 1, LN
            CALL SORT ( pdf3 (1, i, j), npts, pdf3 (1, i, j), iret )
c           Following line replaces an "EQUIVALENCE" statement
c           that was used in original code
cc            pdf3(1, 1, 4)=pdf(1, 1, 1)
         END DO
      END DO
            
      
C     Smooth the 40N and 40S f's, then compute the ratio tmpi/tovs.
C     Compute north and south f ratios and load into 50-90 deg polar
C     cap.  Do a smooth-fill between 40 and 50 (and uselessly over
C     40N-S), then use these with the PDF to fill in the global field 
C     of f outside 40N-S.  Start by smoothing the tmpif in the 40N-S
C     zone.
C     
      DO  j = 51, LT-50
         DO  i = 1, LN
            IF  ( ntmpi (i, j) .GT. 0 )  THEN
               tmpif (i, j) = tmpif (i, j) / ntmpi (i, j)
            ELSE
               tmpif (i, j) = RMISS
            END IF
         END DO
      END DO
      CALL SMOOTH  ( nsm, smth, 3.0, tmpif (1, 51), LN, LT-100,
     +     tmpif (1, 51), iret )
      IF  ( iret .NE. 0 )  THEN
         WRITE (*, *) 'Error', iret, ' in SMOOTH'
         STOP
      END IF
C     
      DO  i = 1, LN
         IF  ( i .EQ. 1 )  THEN
            iw = LN
            ie = i + 1
         ELSE IF  ( i .EQ. LN )  THEN
            iw = i - 1
            ie = 1
         ELSE
            iw = i - 1
            ie = i + 1
         END IF
         IF  ( tovsf40n (iw) + tovsf40n (i) + tovsf40n (ie) .GT. 0. )
     +        ratf (i, 51   ) = ( tmpif40n (iw) + tmpif40n (i) + 
     +        tmpif40n (ie) ) / 
     +        ( tovsf40n (iw) + tovsf40n (i) + 
     +        tovsf40n (ie) )
         IF  ( tovsf40s (iw) + tovsf40s (i) + tovsf40s (ie) .GT. 0. )
     +        ratf (i, LT-50) = ( tmpif40s (iw) + tmpif40s (i) + 
     +        tmpif40s (ie) ) / 
     +        ( tovsf40s (iw) + tovsf40s (i) + 
     +        tovsf40s (ie) )
      END DO
      IF  ( tovsfn .GT. 0. )  THEN
         ratn = tmpifn / tovsfn
      ELSE
         ratn = RMISS
      END IF
      IF  ( tovsfs .GT. 0. )  THEN
         rats = tmpifs / tovsfs
      ELSE
         rats = RMISS
      END IF
      WRITE (*, *) '37-40N avg tovsf, tmpif, ratio =', 
     +     tovsfn, tmpifn, ratn
      WRITE (*, *) '37-40S avg tovsf, tmpif, ratio =', 
     +     tovsfs, tmpifs, rats
      DO  j = 1, 40
         DO  i = 1, LN
            ratf (i, j          ) = ratn
            ratf (i, LT - j + 1 ) = rats
         END DO
      END DO
      npass = -1
      CALL FILLSM ( ratf, LN, LT, LN, 0, npass, ratf, iret )
C     
      DO  j = 1, 50
         DO  i = 1, LN
            DO  m = 1, npts
               IF  ( pdf3 (m, i, j) .GE. 0. )  THEN
                  npdf (i, j) = npts - m + 1
                  GO TO 100
               END IF
            END DO
            npdf  (i, j) = 0
            tmpif (i, j) = RMISS
            GO TO 200
 100        CONTINUE
            DO  m = npts - npdf (i, j) + 1, npts
               IF  ( pdf3 (m, i, j) .GT. 0. )  GO TO 110
            END DO
            m = npts
 110        tmpif (i, j) = ratf (i, j) * ( npts - m ) / npdf (i, j)
 200        CONTINUE
         END DO
      END DO
      DO  j = LT-49, LT
         DO  i = 1,  LN
            DO  m = 1, npts
               IF  ( pdf3 (m, i, j) .GE. 0. )  THEN
                  npdf (i, j) = npts - m + 1
                  GO TO 300
               END IF
            END DO
            npdf  (i, j) = 0
            tmpif (i, j) = RMISS
            GO TO 400
 300        CONTINUE
            DO  m = npts - npdf (i, j) + 1, npts
               IF  ( pdf3 (m, i, j) .GT. 0. )  GO TO 310
            END DO
            m = npts
 310        tmpif (i, j) = ratf (i, j) * ( npts - m ) / npdf (i, j)
 400        CONTINUE
         END DO
      END DO
      
      

C     Wrap up getting the number of valid PDF points for each box by
C     working on the 40N-S band (computed previously poleward of that).
C     Compute TOVS revised zero point and average reduced TOVS.  
C     Points with "no PDF" data are set to missing and smooth-filled 
C     below.  The (rounded) index of the lowest non-zero TOVS rate 
C     that's allowed is m.  If it's npts, there's only 0 or 1 value
C     allowed to be non-zero, implying that the calculation is 
C     unstable, so set to RMISS and smooth-fill below.  If it's all 
C     points (npts-npdf or npts-npdf+1), all rates are allowed and 
C     z_pt is automatically zero.  For all other points, z_pt is the 
C     value of the next smaller (sorted) pdf value.  When z_pt is non-
C     missing (last 2 cases), accumulate the TOVS excesses over the 
C     zero point value for allowed non-zero rates and then average.
C     Note that it is possible for some of the allowed non-zero values
C     to have the z_pt value and therefore contribute zero.
C     
      DO  j = 51, LT-50
         DO  i = 1,  LN
            DO  m = 1, npts
               IF  ( pdf3 (m, i, j) .GE. 0. )  THEN
                  npdf (i, j) = npts - m + 1
                  GO TO 450
               END IF
            END DO
            npdf (i, j) = 0
 450        CONTINUE
         END DO
      END DO
C     

      
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( npdf (i, j) .LE. 0 )  THEN
               z_pt  (i, j) = RMISS
               redto (i, j) = RMISS
c     Added by Sapiano to avoid segfault
            elseif ( tmpif (i, j) .EQ. RMISS )  THEN
               z_pt  (i, j) = RMISS
               redto (i, j) = RMISS
               print *, "Added by Sapiano; tmpif(",i,",",j,")=", 
     +              tmpif(i,j)
               print *, "Set z_pt and redto to missing to "//
     +              "avoid segfault"
            ELSE
               m = NINT ( npts - npdf (i, j) * tmpif (i, j) )
               IF  ( m .EQ. npts )  THEN
                  z_pt  (i, j) = RMISS
                  redto (i, j) = RMISS
               ELSE
                  IF  ( m .LE. npts - npdf (i, j) + 1 )  THEN
                     z_pt (i, j) = 0.
                  ELSE
                     z_pt (i, j) = pdf3 (m-1, i, j)
                  END IF
                  DO  m = npts - npdf (i, j) + 1, npts
                     IF  ( pdf3 (m, i, j) .GT. z_pt (i, j) )
     +                    redto (i, j) = redto (i, j) + pdf3 (m, i, j) - 
     +                    z_pt  (i, j)
                  END DO
                  IF  ( redto (i, j) .GT. 0. )  THEN
                     redto (i, j) = redto (i, j) / npdf (i, j)
                  ELSE
                     redto (i, j) = redto (i, j) / npdf (i, j)
                  END IF
               END IF
            END IF
         END DO
      END DO
      
      
C     Convert the 2.5x2.5-deg monthly global precip (in mm/d)
C     to 1x1-deg monthly on the tropical strip (still mm/hr).
      CALL BILIN ( p_mon, 144,    72, 0., 0.,   360, 180, 2.5, 
     +     2.5,   .TRUE., .TRUE., pcum, iret )
      
C     Get the scaling ratio between the monthly accumulation and
C     the PDF accumulation and do a smooth-fill to ensure values
C     everywhere.  The MIN prevents unreasonable behavior in 
C     sparse-rain areas; 4 is chosen by inspecting 199801.
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( redto (i, j) .GT. 0. )  THEN
               rat (i, j) = AMIN1 ( pcum (i, j) / redto (i, j), 4. )
            ELSE
               rat (i, j) = RMISS
            END IF
         END DO
      END DO
      npass = -1
      CALL FILLSM  ( rat, LN, LT, LN, 0, npass, rat, iret )
      
      
C     Dump out fields.
      fname=fbas(1:l_fbas)//'.pcum'
      CALL Write_Binary_File(fname, pcum, LN, LT)
      fname=fbas(1:l_fbas)//'.adjf'
      CALL Write_Binary_File(fname, tmpif, LN, LT)
      fname=fbas(1:l_fbas)//'.zpt'
      CALL Write_Binary_File(fname, z_pt, LN, LT)
      fname=fbas(1:l_fbas)//'.trat'
      CALL Write_Binary_File(fname, rat, LN, LT)
      fname=fbas(1:l_fbas)//'.ratf'
      CALL Write_Binary_File(fname, ratf, LN, LT)
      
      
      
C     Now read the TOVS data and compute the proxy.  Note: the fill
C     value for missing TOVS is -999.9.  A missing z_pt appears to 
C     indicate no rain in the month, so it flags zero rain.  Fill
C     TOVS to account for holes in the forward-regridding 
C     (note: into redto) using a latitude-dependent scheme.  Note
C     that this preserves large gaps in the data (e.g., September 
C     11-18, 2001).  Use tovs to check the filled values in redto 
C     and zero "very small" (<.05, chosen as rounding to 0 to the 
C     nearest .1, as in FUSE2.F) values.  Wrap up by dumping the 
C     field.
      DO  m = 1, nto

         CALL Read_Binary_File (toname(m), 360, 180, 1, tovs)
         if (index(toname(m),'.utime') .NE. 0) then  
            CALL SWAPR4 (tovs, 360, 180)
         end if



         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( tovs (i, j) .GT. 0. )  THEN
                  IF  ( z_pt (i, j) .EQ. RMISS )  THEN
                     tovs (i, j) = 0.
                  ELSE IF  ( tovs (i, j) .GT. z_pt (i, j) )  THEN
                     tovs (i, j) = ( tovs (i, j) - z_pt (i, j) ) *
     +                    rat (i, j)
                  ELSE
                     tovs (i, j) = 0.
                  END IF
               ELSE IF  ( tovs (i, j) .LT. 0. )  THEN
                  tovs (i, j) = RMISS
               END IF
            END DO
         END DO
         
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( tovs (i, j) .EQ. RMISS .AND. 
     +              redto (i, j) .LT. .05 )  redto (i, j) = 0.
            END DO
         END DO

         CALL LFILL (tovs, LN, LT, tovsl)
         

 500     FORMAT ( I2.2 )
         
         
c        Output data  
         tmpname=toname(m)
         idx=index(tmpname,'.avg')
         if (idx .EQ. 0) then
            idx=index(tmpname,'.utime')
         end if
         dd=tmpname(idx-2:idx-1)
         fname=fbas(1:l_fbas)//dd//'.t_fsg'
         CALL Write_Binary_File(fname,tovsl, LN, LT)
         
      END DO
C     
      STOP
      
      
      end program Main_GPCPcdr_d4_corr_g
      




