	SUBROUTINE LFILL (ain, LN, LT, aout)
C
C***********************************************************************
C* LFILL	Fills holes in a field using latitude weighting        *
C*								       *
C* Called by:	CORR_T						       *
C*   								       *
C* Calls:	<none>						       *
C*  								       *
C* LFILL fills holes in the input array using a 3-by-3 box average at  *
C* the equator.  The longitudinal expanse of the box expands with      *
C* increasing latitude (as a function of cosine) to compensate for the *
C* smaller east-west area coverage.  Note that LFILL DOES NOT perform  *
C* a smooth fill.  Any gridboxes that have no data inside or           *
C* immediately surrounding will remain missing.  This subroutine was   *
C* originally developed for the 1DD TOVS data to preserve large gaps   *
C* in mid-latitudes but fill in at appropriately at the poles.	       *
C*  								       *
C* Current Restrictions:					       *
C*  								       *
C* (1) The input array must be global				       *
C* (2) The size of the smoother is set at 3-by-3 boxcar	(at equator)   *
C*  								       *
C* Input Parameters:						       *
C*								       *
C*  ain		REAL	input array to be smooth-filled		       *
C*  LN		INT	longitudinal size of input array	       *
C*  LT		INT	latitudinal size of input array		       *
C* 								       *
C* Output Parameters:						       *
C*								       *
C*  aout	REAL	output filled array			       *
C* 								       *
C**								       *
C* Log:								       *
C* D.Bolvin/SSAI	 01/02	First Version			       *
C***********************************************************************
C
	IMPLICIT	NONE
C
	INTEGER*4	LNM, LTM
	PARAMETER	( LNM = 1440,
     +			  LTM =  720 )
C
     	INTEGER*4	LN, LT, RMISS/-99999.0/, FILLCHK/-1.0/
	INTEGER*4	i, j, k, l, icount, icountn, icounts, 
     +			lim, m
C
	REAL*4		alchk, sum, sumn, sums, res
	REAL*4		ain(LN,LT), aout(LN,LT)
	REAL*4		work(LNM+LNM,LTM)


c       Need to convert degrees to radians - SAPIANO                                                               
        REAL deg2rad
        deg2rad = 4*ATAN(1.0)/180


C
C***********************************************************************
C	Ensure the input array does not exceed the maximum size.
C
	IF  ((LN .GT. LNM) .OR. (LT .GT. LTM)) THEN
	    WRITE (*,*) 'ERROR: INPUT ARRAY SIZE LARGER THAN PRESET ',
     +			'MAXIMUM; TERMINATION EXECUTION; SUBROUTINE ',
     +			'LFILL'
     	    STOP
	END IF
C
C***********************************************************************
C	Ensure the input array is global.
C
	alchk = FLOAT (LN) / FLOAT (LT)
C
	IF  (alchk .NE. 2.0) THEN
	    WRITE (*,*) 'ERROR: INPUT ARRAY IS NOT GLOBAL; ',
     +			'TERMINATION EXECUTION; SUBROUTINE LFILL'
     	    STOP
	END IF
C
C***********************************************************************
C	Determine the resolution of the input array.  This will be used
C	to compute the cosine of the latitude from the array indices.
C
	res = 360.0 / FLOAT (LN)
C
C***********************************************************************
C	Fill in the working array to make indexing across the east-west
C	boundaries easier to manage.
C
	DO i = 1, LN
	    DO j = 1, LT
	    	work(i + LN/2,j) = ain(i,j)
	    END DO
	END DO
C
	DO i = 1, LN/2
	    DO j = 1, LT
	    	work(i,j) = work(i + LN,j)
	    	work(i + 1.5*LN,j) = work(i + LN/2,j)
	    END DO
	END DO
C
C***********************************************************************
C	Do the fill.  The east-west extent of the boxes is determined by
C	the cosine of the latitude.
C
	DO j = 2, LT - 1
C
	    lim = MIN0 (NINT(((3.0/COS(((90.0 - (res / 2.0)) - 
     +			((j - 1) * res)) * deg2rad)) - 1.0) / 2.0), LN/2)
C
	    DO i = 1, LN
C
	    	sum = 0.0
	    	icount = 0
C
	    	DO k = j - 1, j + 1
C
	    	    IF  (k .EQ. j) THEN
	    	    	m = lim
	    	    ELSE
	    	    	m = 1
	    	    END IF
C
	    	    DO l = i - m + LN/2, i + m + LN/2
	    	    	IF  (work(l,k) .GT. FILLCHK) THEN
	    	    	    sum = sum + work(l,k)
	    	    	    icount = icount + 1
	    	    	END IF
	    	    END DO
C
	    	END DO
C
	    	IF  (icount .GT. 0) THEN
	    	    aout(i,j) = sum / FLOAT (icount)
	    	ELSE
	    	    aout(i,j) = RMISS
	    	END IF
C
	    END DO
C
	END DO
C
C***********************************************************************
C	Do the special case where the fill is done at the poles. Do the
C	north pole first.
C
	sum = 0.0
	icount = 0
C
	DO i = 1, LN
	    IF  (work (i + LN/2,1) .GT. FILLCHK) THEN
	    	sum = sum + work(i + LN/2,1)
	    	icount = icount + 1
	    END IF
	END DO
C
	DO i = 1, LN
C
	    sumn = 0.0
	    icountn = 0
C
	    DO l = i + LN/2 - 1 , i + LN/2 + 1
	    	IF  (work (l,2) .GT. FILLCHK) THEN
	    	    sumn = sumn + work(l,2)
	    	    icountn = icountn + 1
	    	END IF
	    END DO
C
	    IF  ((icount + icountn) .GT. 0) THEN
	    	aout(i,1) = (sum + sumn) / FLOAT (icount + icountn)
	    ELSE
	    	aout(i,1) = RMISS
	    END IF
C
	END DO
C
C***********************************************************************
C	Do the south pole.
C
	sum = 0.0
	icount = 0
C
	DO i = 1, LN
	    IF  (work (i + LN/2,LT) .GT. FILLCHK) THEN
	    	sum = sum + work(i + LN/2,LT)
	    	icount = icount + 1
	    END IF
	END DO
C
	DO i = 1, LN
C
	    sums = 0.0
	    icounts = 0
C
	    DO l = i + LN/2 - 1 , i + LN/2 + 1
	    	IF  (work (l,LT - 1) .GT. FILLCHK) THEN
	    	    sums = sums + work(l,LT - 1)
	    	    icounts = icounts + 1
	    	END IF
	    END DO
C
	    IF  ((icount + icounts) .GT. 0) THEN
	    	aout(i,LT) = (sum + sums) / FLOAT (icount + icounts)
	    ELSE
	    	aout(i,LT) = RMISS
	    END IF
C
	END DO
C
C***********************************************************************
C	Fill in the original field with smoothed values where they exist.
C
	DO i = 1, LN
	    DO j = 1, LT
	    	IF  (work(i + LN/2,j) .GT. FILLCHK) THEN
	    	    aout(i,j) = work(i + LN/2,j)
	    	END IF
	    END DO
	END DO
C
	RETURN
	END
	
